package figure;

import java.awt.Point;
import java.awt.geom.Point2D;

public class Circle extends Figure {
	private Point2D[] vertices = null; // 
	private double[] sides = null; // 
	private double[] angles = null; // 
	private Bound bound = null; // ٿڽ

	private Point center = null;
	private double radius = 0.0;
	private int step = 0;
	
	// default constructor
	public Circle() {
	}

	// constructor
	public Circle(Point center, double radius, int step) {
		this.center = center;
		this.radius = radius;
		this.step = step;
		setVertices(createVertices(center, radius, step)); 
	}
	
	// circle points
	public static Point2D[] createVertices(Point center, double radius, int step) {
		Point2D[] points = new Point2D[step];
		double rad = 2 * Math.PI / step;
		for (int i=0; i<step; i++) {
			points[i] = new Point2D.Double(center.getX() + radius * Math.cos(i*rad), center.getY() + radius * Math.sin(i*rad));
		}
		System.out.println("points: " + java.util.Arrays.toString(points));
		return points;
	}

	public void setVertices(Point2D[] vertices) {
		this.vertices = vertices;
        //System.out.println("Point " + java.util.Arrays.toString(points));
    	updateBound(vertices);
    	updateSides(vertices);
    	updateAngles(vertices);
    }
 	
    public void updateBound(Point2D[] points) {
		this.bound = Bound.findBound(points);
		//System.out.println(bound);
    }
    
    public void updateSides(Point2D[] points) {
		if (points == null) return;
		this.sides = new double[points.length];
		for (int i=0; i<points.length; i++) {
			int nextIndex = (i + 1) % points.length;
			this.sides[i] = points[i].distance(points[nextIndex]);
		}
		//System.out.println("Side " + java.util.Arrays.toString(sides));
    }
    
    public void updateAngles(Point2D[] points) {
    	if (points == null) return;
		this.angles = new double[points.length];
		for (int i=0; i<points.length; i++) {
			int prevIndex = i==0 ? (i + points.length - 1) : (i - 1);
			int nextIndex = (i + 1) % points.length;
			this.angles[i] = Vector2D.innerAngle(points[prevIndex], points[i], points[nextIndex]);
		}
		//System.out.println("Angle " + java.util.Arrays.toString(angles));
    }

	// getter
	public double getRadius() {
		return radius;
	}

	// setter
	public void setRadius(double radius) {
		this.radius = radius;
	}

	// circle diameter
	public double diameter() {		
		return 2 * radius;
	}
	
	// circle area
	public double area() {
		return area(radius);
	}

	// circle circumference
	public double perimeter() {
		return perimeter(radius);
	}

	// circle area = Math.PI * radius * radius
	public static double area(double radius) {
		return Math.PI * radius * radius;
	}

	// circle circumference = 2 * Math.PI * radius
	public static double perimeter(double radius) {
		return 2 * Math.PI * radius;
	}

	@Override
	public String toString() {
		return "Circle [radius=" + radius + ", diameter()=" + diameter() + ", area()=" + area() + ", perimeter()="
				+ perimeter() + "]";
	}

	public void getAdditionalUserInput() {
		System.out.print("please input circle center Point [10.0 10.0] : "); 
		center = UserInput.getPoint();
		System.out.print("please input circle radius [7.5]: "); 
		radius = UserInput.getDouble();
		System.out.print("please input circle # of step [16]: "); 
		step = UserInput.getInteger();
		setVertices(createVertices(center, radius, step));
	}
}
